//
//  KLCodeResignView.m
//  codeview
//
//  Created by mac2022 on 2023/11/23.
//

#import "KLCodeResignView.h"

#define WEAKSELF typeof(self) __weak weakSelf = self;
//自定义 验证码展示视图 view，由一个label和一个下划线组成
@interface KLCodeView : UIView

@property (strong, nonatomic) NSString *text;
@property (strong, nonatomic) UILabel *codeLabel;
@property (strong, nonatomic) UIView *lineView;

@end


@interface KLCodeResignView () <UITextFieldDelegate>

@property (strong, nonatomic) UITextField *contentText; //监听内容输入
@property (strong, nonatomic) NSArray<KLCodeView *> *codeViewsArr;//显示输入内容的codeView数组
@property (assign, nonatomic) NSInteger currIndex;//当前待输入的codeView的下标


@end

@implementation KLCodeResignView

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    self.backgroundColor = [UIColor whiteColor];
    if (self) {
        
        WEAKSELF
        [self addSubview:self.contentText];
        self.contentText.frame = CGRectMake(0, 0, CGRectGetWidth(weakSelf.frame), CGRectGetHeight(weakSelf.frame));
    }
    
    return self;
}

-(void)setCodeBits:(NSInteger)codeBits{
    _codeBits = codeBits;
    
    for(NSInteger i = 0; i < self.codeBits; i++) {
        KLCodeView *codeView = self.codeViewsArr[i];
        [self addSubview:codeView];
    }
}


#pragma mark --- UITextField delegate
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    //完成 则收回键盘
    if ([string isEqualToString:@"\n"]) {
        [textField resignFirstResponder];
        return NO;
    }
    //删除 操作
    if ([string isEqualToString:@""]) {
        if (self.currIndex == 0) {//待输入的下标为0时 删除时下标不变化，否则下标减1
            self.codeViewsArr[self.currIndex].text = string;
        } else {
            self.codeViewsArr[--self.currIndex].text = string;
            if (self.codeResignUnCompleted) {
                NSString *content = [textField.text substringToIndex:self.currIndex];
                self.codeResignUnCompleted(content);
            }
        }
        return YES;
    }
    //判断 输入的是否是纯数字，不是纯数字 输入无效
    if (![self judgePureInt:string]) {
        return NO;
    }
    //如果输入的内容超过了验证码的长度 则输入无效
    if ((textField.text.length + string.length) > self.codeBits) {
        return NO;
    }
    //输入的数字，则当前待输入的下标对应的 view中添加输入的数字，并且下标加1
    self.codeViewsArr[self.currIndex++].text = string;
    //当当前待输入的下标为codebits时表示已经输入了对应位数的验证码，执行完成操作
    if (self.currIndex == self.codeBits && self.codeResignCompleted) {
        NSString *content = [NSString stringWithFormat:@"%@%@", textField.text, string];
        self.codeResignCompleted(content);
    } else {
        if (self.codeResignUnCompleted) {
            NSString *content = [NSString stringWithFormat:@"%@%@", textField.text, string];
            self.codeResignUnCompleted(content);
        }
    }
    
    return YES;
}

- (UITextField *)contentText {
    if (!_contentText) {
        _contentText = [[UITextField alloc] init];
        //背景颜色和字体颜色都设置为透明的，这样在界面上就看不到
        _contentText.backgroundColor = [UIColor clearColor];
        _contentText.textColor = [UIColor clearColor];
        _contentText.keyboardType = UIKeyboardTypeNumberPad;//数字键盘
        _contentText.returnKeyType = UIReturnKeyDone;//完成
        _contentText.tintColor = [UIColor clearColor];//设置光标的颜色
        _contentText.delegate = self;
    }
    return _contentText;
}


- (NSArray<KLCodeView *> *)codeViewsArr {
    if (!_codeViewsArr) {
        NSMutableArray *arr = [NSMutableArray array];
        for (NSInteger i = 0; i < self.codeBits; i++) {
            CGFloat codeViewWidth = self.bounds.size.width/(self.codeBits * 1.2-0.2);
            CGFloat left = codeViewWidth * 1.2 * i;
                        
            KLCodeView *codeView = [[KLCodeView alloc] initWithFrame:CGRectMake(left, 0, codeViewWidth, CGRectGetHeight(self.contentText.frame))];
            [arr addObject:codeView];
        }
        _codeViewsArr = [NSArray arrayWithArray:arr];
    }
    return _codeViewsArr;
}


//判断一个字符串是都是纯数字
- (BOOL)judgePureInt:(NSString *)content {
    NSScanner *scan = [NSScanner scannerWithString:content];
    int val;
    return [scan scanInt:&val] && [scan isAtEnd];
}

@end


@implementation UITextField (ForbiddenSelect)

/*
 该函数控制是否允许 选择 全选 剪切 f粘贴等功能，可以针对不同功能进行限制
 返回YES表示允许对应的功能，返回NO则表示不允许对应的功能
 直接返回NO则表示不允许任何编辑
 */
- (BOOL)canPerformAction:(SEL)action withSender:(id)sender {
    return NO;
}

@end


//验证码展示视图 的实现
@implementation KLCodeView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        WEAKSELF
        self.backgroundColor = [UIColor whiteColor];
        self.userInteractionEnabled = NO;
        //数字编码 label
        _codeLabel = [[UILabel alloc] init];
        _codeLabel.textColor = [UIColor colorWithRed:204.0/255 green:44.0/255 blue:50.0/255 alpha:1];
        _codeLabel.font = [UIFont systemFontOfSize:25];
        _codeLabel.textAlignment = NSTextAlignmentCenter;
        _codeLabel.backgroundColor = [UIColor clearColor];
        [self addSubview:_codeLabel];
        _codeLabel.frame = CGRectMake(0, 0, CGRectGetWidth(weakSelf.frame), CGRectGetHeight(weakSelf.frame)-10);
        
        _lineView = [[UIView alloc] init];
        _lineView.backgroundColor = [UIColor colorWithRed:221.0/255 green:221.0/255 blue:221.0/255 alpha:1];
        [self addSubview:_lineView];
        _lineView.frame = CGRectMake(0, CGRectGetMaxY(weakSelf.frame)-2, CGRectGetWidth(weakSelf.frame), 2);
    }
    return self;
}

- (void)setText:(NSString *)text {
    if (text.length > 0) {
        self.codeLabel.text = [text substringToIndex:1];//只取一位数
        self.lineView.backgroundColor = [UIColor colorWithRed:204.0/255 green:44.0/255 blue:50.0/255 alpha:1];
    } else {
        self.codeLabel.text = @"";
        self.lineView.backgroundColor = [UIColor colorWithRed:221.0/255 green:221.0/255 blue:221.0/255 alpha:1];
    }
}



@end

